#pragma once

#include "glm/glm.hpp"
#include "glm/gtc/matrix_transform.hpp"

#define _USE_MATH_DEFINES
#include <math.h>

class Camera
{
private:
	//Private Attributes
	glm::vec2 m_AngleNew;
	glm::bvec4 m_KeysPress;
	glm::vec3 m_Forward;
	glm::vec3 m_Left;
	glm::ivec2 m_OldPositionMouse;
	bool m_MouseClick;
public:
	// Public Attributes
	glm::vec3 Position;
	glm::vec3 Target;
	glm::vec3 Up;
	float MoveFactor;
	float FoV;
	float Ratio;
	float Near;
	float Far;

	Camera(const glm::vec3& position, float fov = 70.f, float ratio = 1.f) :
		m_AngleNew(0.f),
		m_KeysPress(false),
		m_MouseClick(false),
		Position(position),
		Target(0,0,0),
		Up(0,1,0),
		MoveFactor(10.0f),
		FoV(fov),
		Ratio(ratio)
	{
	}

	void ComputeAngles()
	{
		double r_temp = cos(-m_AngleNew.x * M_PI / 180);
		m_Forward.y = sin(-m_AngleNew.x * M_PI / 180);
		m_Forward.x = r_temp * cos(m_AngleNew.y * M_PI / 180);
		m_Forward.z = r_temp * sin(m_AngleNew.y * M_PI / 180);

		m_Left = glm::core::function::geometric::cross(Up, m_Forward);
		m_Left = glm::normalize(m_Left);
	}
	void Update(float deltaTime)
	{
		const float factor = (deltaTime * MoveFactor);
		if(m_KeysPress.x)
                        Position -= (factor*m_Forward);
		if(m_KeysPress.y)
                        Position += m_Forward*factor;
		if(m_KeysPress.z)
                        Position -= m_Left*factor;
		if(m_KeysPress.w)
                        Position += m_Left*factor;
		ComputeAngles();
	}

	glm::mat4x4 GetView()
	{
		return glm::lookAt(Position,Target,Up);
	}

	glm::mat4x4 GetProjection()
	{
		return glm::perspectiveFov(FoV, Ratio, 1.f, Near, Far);
	}
};

std::ostream& operator<<(std::ostream& out, const glm::vec3& v)
{
	out << "<" << v.x << "; " << v.y << "; " << v.z << ">";
	return out;
}

std::ostream& operator<<(std::ostream& out, const Camera& c)
{
	out << "(Position : " << c.Position << " | Target : " << c.Target << ")";
	return out;
}
