#include "Engine.hpp"

#include <iostream>

#include <SDL/SDL.h>

#include "Window.hpp"
#include "GLWindow.hpp"

Engine :: Engine(EngineAPI api, unsigned int width, unsigned int height, bool isFullScreen, const std::string& windowName):apis(api),pWindow(NULL),lastFrameTime(0)
{
    if ( (api & EAPI_SDL) == EAPI_SDL )
    {
        unsigned int flags = SDL_INIT_VIDEO;
        if ( (api & EAPI_SDL_Mixer) == EAPI_SDL_Mixer )
        {
            flags |= SDL_INIT_AUDIO;
        }

        SDL_Init(flags);

        if ( (api & EAPI_SDL_Mixer) == EAPI_SDL_Mixer )
        {
            // Mix_Init;
        }
    }

    if ( isFullScreen )
    {
        pWindow = new GLWindow(width, height, windowName, true);
    }
    else
    {
        pWindow = new GLWindow(width, height, windowName, false);
    }

    if ( pWindow == NULL )
    {
        std::cerr << "Fail to allocate Window memory";
        throw std::bad_alloc();
    }

    if ( (api & EAPI_OpenGL) == EAPI_OpenGL )
    {

    }

    std::cout << "NEngine initiliased" << std::endl;
}

Engine :: ~Engine()
{
    if ( (apis & EAPI_SDL_Mixer) == EAPI_SDL_Mixer )
    {
        // Mix_Quit();
    }

    if ( (apis & EAPI_SDL) == EAPI_SDL )
    {
        SDL_Quit();
    }

    std::cout << "NEngine closed" << std::endl;
}

bool Engine :: checkExtension(const char* extensionName)
{

    return true;
}

void Engine :: waitFrame()
{
    unsigned int actualTime = SDL_GetTicks();

    if ( actualTime - lastFrameTime < 15 )
    {
        SDL_Delay(15 - (actualTime - lastFrameTime));
    }

    lastFrameTime = SDL_GetTicks();
}

unsigned int Engine :: getTime()
{
    return SDL_GetTicks();
}
