#ifndef __ENGINE_HPP__
#define __ENGINE_HPP__

#include <string>
#include <exception>

class Window;

class EngineInitialisationException : public std::exception
{
    private:
        std::string message;

    public:
        EngineInitialisationException(const std::string& message):message(message) {}
        virtual ~EngineInitialisationException()throw() {}

        virtual const char* what() const throw()
        {
            return message.c_str();
        }
};

typedef unsigned int EngineAPI;
const EngineAPI EAPI_OpenGL = 1;
const EngineAPI EAPI_SDL = 2;
const EngineAPI EAPI_SDL_Mixer = 4;
const EngineAPI EAPI_SDL_Image = 8;


class Engine
{
    private:

        EngineAPI apis;

        Window* pWindow;

        unsigned int lastFrameTime;

    public:

        Engine(EngineAPI api, unsigned int width, unsigned int height, bool isFullScreen, const std::string& windowName);
        ~Engine();

        Window* getWindow() { return pWindow; }

        bool checkExtension(const char* extensionName);

        void waitFrame();
        unsigned int getTime();
};

#endif
