#include "Window.hpp"

#include <SDL/SDL.h>

Window :: Window(unsigned int width, unsigned int height, const std::string& windowName, unsigned int sdlFlags)
{
    if ( (sdlFlags & SDL_FULLSCREEN) != SDL_FULLSCREEN )
    {
        sdlFlags |= SDL_RESIZABLE;
    }

    pWindow = SDL_SetVideoMode(width,height,32,sdlFlags);
    if ( pWindow == NULL )
    {
        throw WindowCreationException("Fail to create the window");
    }

    SDL_ShowCursor(SDL_DISABLE);
    SDL_WM_SetCaption(windowName.c_str(),NULL);
}

Window :: ~Window()
{
    // Not ... since SDL does everything
}

unsigned int Window :: getWidth()const { return pWindow->w; }
unsigned int Window :: getHeight()const { return pWindow->h; }

void Window :: clear()
{
    SDL_FillRect(pWindow,NULL,SDL_MapRGB(pWindow->format,0,0,0));
}

void Window :: resize(unsigned int width, unsigned int height)
{
    pWindow = SDL_SetVideoMode(width,height,32,pWindow->flags);
    if ( pWindow == NULL )
    {
        throw WindowCreationException("Fail to resize");
    }
}

void Window :: refresh()
{
    SDL_UpdateRect(pWindow,0,0,0,0);
}
