#define GL3_PROTOTYPES 1
#include "GL/gl3.h"
#include "Engine.hpp"
#include "Controllers/InputManager.hpp"
#include "Controllers/SDL_Keyboard.hpp"
#include "Window.hpp"
#include <libmodplug/modplug.h>
#include <stdio.h>
#include <stdlib.h>
#include "SDL/SDL.h"
#include "SDL/SDL_audio.h"
#include <fstream>
#include <sstream>
#include "libmodplug/modplug.h"
#include "Camera.h"
#include <glm/gtc/matrix_inverse.hpp>
#include <glm/gtc/type_ptr.hpp>
#include "math.h"
#define max(a,b) (a>b)?a:b


ModPlugFile *Module = NULL;
int Read_Length;

void mixaudio(void *unused,Uint8 *stream,int len)
{
    Read_Length=ModPlug_Read(Module,(Uint8*)stream,len);
    if(Read_Length==0) ModPlug_Seek(Module,0);
}

ModPlugFile * Load_Module(char *FileName)
{
    ModPlugFile *TmpModule       =NULL;
    FILE        *FilePointer  =NULL;
    char        *Module_Memory=NULL;
    unsigned int Module_Size  =0;
    unsigned int File_Read    =0;
    FilePointer=fopen(FileName,"rb");
    if(FilePointer==NULL) return NULL;


    /*This here is used to get the complete File length of the Module */
    fseek(FilePointer, 0L, SEEK_END);
    Module_Size = ftell(FilePointer);
    fseek(FilePointer, 0L, SEEK_SET);

    /*We are Allocating Memory for the loaded Module, added one byte for safety*/
    Module_Memory=(char *)malloc(Module_Size*sizeof(char)+1);
    if(Module_Memory==NULL) return NULL;

    /*Reading the Complete Module to Memory*/
    File_Read=fread(Module_Memory,1,Module_Size,FilePointer);
    if(File_Read==-1)
    { free(Module_Memory); return NULL;}

    /* And Here we are passing the Loaded Module to Modplug, it needs the Memory Location and the Module Size
   Modplug will return a Valid ModPlugFile Structure*/
    TmpModule=ModPlug_Load((void*)Module_Memory,Module_Size);
    if(TmpModule==NULL)
    {free(Module_Memory); return NULL;}

    /* Modplug keeps the loaded Module in it's own allocated Memory so we can release our allocated memory,
  or you can keep it, for later use */
    free(Module_Memory);
    /* Here the ModplugFile Structure is returned */
    return TmpModule;
}

GLuint CreateShader(GLenum eShaderType, const std::string &strShaderFile)
{
        GLuint shader = glCreateShader(eShaderType);
        const char *strFileData = strShaderFile.c_str();
        glShaderSource(shader, 1, &strFileData, NULL);

        glCompileShader(shader);

        GLint status;
        glGetShaderiv(shader, GL_COMPILE_STATUS, &status);
        if (status == GL_FALSE)
        {
                GLint infoLogLength;
                glGetShaderiv(shader, GL_INFO_LOG_LENGTH, &infoLogLength);

                GLchar *strInfoLog = new GLchar[infoLogLength + 1];
                glGetShaderInfoLog(shader, infoLogLength, NULL, strInfoLog);

                const char *strShaderType = NULL;
                switch(eShaderType)
                {
                case GL_VERTEX_SHADER: strShaderType = "vertex"; break;
                case GL_GEOMETRY_SHADER: strShaderType = "geometry"; break;
                case GL_FRAGMENT_SHADER: strShaderType = "fragment"; break;
                }
                delete[] strInfoLog;
        }

        return shader;
}

GLuint CreateProgram(const std::vector<GLuint> &shaderList)
{
        GLuint program = glCreateProgram();

        for(size_t iLoop = 0; iLoop < shaderList.size(); iLoop++)
                glAttachShader(program, shaderList[iLoop]);

        glLinkProgram(program);

        GLint status;
        glGetProgramiv (program, GL_LINK_STATUS, &status);
        if (status == GL_FALSE)
        {
                GLint infoLogLength;
                glGetProgramiv(program, GL_INFO_LOG_LENGTH, &infoLogLength);

                GLchar *strInfoLog = new GLchar[infoLogLength + 1];
                glGetProgramInfoLog(program, infoLogLength, NULL, strInfoLog);
                delete[] strInfoLog;
        }

        return program;
}

GLuint theProgram;

void InitializeProgram(const std::string vs, const std::string fs)
{
        std::vector<GLuint> shaderList;

        shaderList.push_back(CreateShader(GL_VERTEX_SHADER, vs));
        shaderList.push_back(CreateShader(GL_FRAGMENT_SHADER, fs));

        theProgram = CreateProgram(shaderList);
}

const float vertexPositions[] = {
    -1.f,  -1.f,  0.f,  1.f,
    -1.f,  1.f,  0.f,  1.f,
    1.f,  1.f,  0.f,  1.f,

    -1.f,  -1.f,  0.f,  1.f,
    1.f,  1.f,  0.f,  1.f,
    1.f,  -1.f,  0.f,  1.f
};

GLuint positionBufferObject;
GLuint vao;


void InitializeVertexBuffer()
{
        glGenBuffers(1, &positionBufferObject);

        glBindBuffer(GL_ARRAY_BUFFER, positionBufferObject);
        glBufferData(GL_ARRAY_BUFFER, sizeof(vertexPositions), vertexPositions, GL_STATIC_DRAW);
        glBindBuffer(GL_ARRAY_BUFFER, 0);
}

//Called after the window and OpenGL are initialized. Called exactly once, before the main loop.
void init(const std::string vs, const std::string fs)
{
  InitializeProgram(vs, fs);
        InitializeVertexBuffer();

        glGenVertexArrays(1, &vao);
        glBindVertexArray(vao);
}

glm::vec3 posCam(float time){
    float t = time/20.;
    glm::vec3 r;
    r.x = 5. * ((4 + cos(3 * t)*cos(2*t)));
    r.z = 5. * ((4 + cos(3 * t)*sin(2*t)));
    r.y = 1.2 + (2 * sin(8*t)) / 4.;
    return r;
}


int main(){

    SDL_Surface *Bitmap=NULL;
    SDL_Surface *Display=NULL;
    SDL_Event    Event;

    SDL_AudioSpec Desire;
    SDL_AudioSpec Obtained;

    ModPlug_Settings settings;

    SDL_Init(SDL_INIT_VIDEO|SDL_INIT_AUDIO);

    Display=SDL_SetVideoMode(0,0,0,SDL_SWSURFACE|SDL_ANYFORMAT);

    Desire.freq=44100;                       /* 44.1 Khz Playback Rate */
    Desire.format=AUDIO_S16SYS;  /* Signed 16Bit Audio format */
    Desire.channels=2;                     /* Stereo, 1 would be mono*/

    Desire.samples=1024;

    Desire.callback=mixaudio;

    Desire.userdata=NULL;

    SDL_OpenAudio(&Desire,&Obtained);

    settings.mFlags=MODPLUG_ENABLE_OVERSAMPLING;
    settings.mChannels = Obtained.channels;
    settings.mBits = 16;
    settings.mFrequency = Obtained.freq;

    settings.mResamplingMode = MODPLUG_RESAMPLE_FIR;

    settings.mLoopCount = -1;

    ModPlug_SetSettings(&settings);

    Module=Load_Module("Dreamzone.mod");
    ModPlug_SetMasterVolume(Module,512);
    SDL_PauseAudio(0);



    Engine e(EAPI_SDL | EAPI_OpenGL, 800,600,1,"A77");

    const std::string strVertexShader = "#version 330 \n layout(location = 0) in vec4 position; out vec2 FragCoord; void main() {FragCoord = position.xy; gl_Position = position;}";

    const std::string strFragmentShader= "#version 330 \n #extension GL_EXT_gpu_shader4 : enable \n #define hsv(h,s,v) mix(vec3(1.), clamp((abs(fract(h+vec3(3., 2., 1.)/3.)*6.-3.)-1.), 0., 1.), s)*v \n out vec4 frag_color; in vec2 FragCoord; in vec3 speed; uniform float time;uniform float view_dist;uniform vec3 camPos;uniform vec2 projMat; uniform mat4 invView;vec2 resolution = vec2(800,600);float smokeCoeff = 0;vec3 smokeColor = hsv(cos(time),-1.,1);float perlin(vec3 p) {vec3 i = floor(p);vec4 a = dot(i, vec3(1., 57., 21.)) + vec4(0., 57., 21., 78.);vec3 f = cos((p-i)*3.14)*(-.5)+.5;a = mix(sin(cos(a)*a),sin(cos(1.+a)*(1.+a)), f.x);a.xy = mix(a.xz, a.yw, f.y);return mix(a.x, a.y, f.z);}float city(vec2 p){   float a = abs(perlin(vec3(p,42.42)));   float result =  mix(0,1.4*max(a-mod(a,.5)+mod(a,.7)-mod(a,.3),0.),max(0.1,min((time - 10) / 5., 1)));   result = mix(result, 0.1, min(max((time-80)/10., 0),1));   return result;}float smoke(vec3 p){  return mix(0, perlin(vec3(p.x + time / 5, p.z + time / 7,6.4 + sin(time/5 + 1.57))) + 0.4, min(1, max(0,(time-20)) / 20.));} vec3 getNormal(vec3 p){    float eps =.1;    return normalize(  vec3(city(p.xz - vec2(eps, 0.0)) - city(p.xz + vec2(eps, 0.0)),  2.0 * eps,  city(p.xz - vec2(0.0, eps)) - city(p.xz + vec2(0.0, eps))));} vec3 sendRay(vec3 origine, vec3 direction, out int id, out vec3 normal, vec3 alpha){ float eps = 0.02; float h = origine.y - 2.; float h2 = (origine+direction).y - 2.; float b=1.; float s = view_dist; float vd = 8; if (mod(time/10,3)<2 &&mod(time/10,3)>=1)     vd = 11;if(h>0)  if(direction.y>=0) {id = 3;normal = vec3(0,0,0); return vec3(0,0,0); }  else b = h/(h2+h);else s = 2.;vec3 currentPosition = origine; for(float t=b; t<vd &&  currentPosition.y<s; t+=eps) {if (currentPosition.y < smoke(currentPosition))         smokeCoeff += 0.01;  currentPosition = origine + t*direction; if( currentPosition.y < city(currentPosition.xz)){ for(float tt = t-eps; tt<t; tt+=eps/10.){currentPosition = origine + tt*direction; if( currentPosition.y < city(currentPosition.xz)) { if( city(currentPosition.xz) == 0.) {id = 0; normal = vec3(0,1,0);}else{id = 1;normal = getNormal(currentPosition);}return currentPosition;}}}}id = 3;normal = vec3(0,0,0); return vec3(0,0,0);}vec3 groundColor(vec3 position){   vec3 color = vec3(1.,1.,1.); if(mod(position.x,1.)<.02 || mod(position.z,1.)<.02 ) color = 1.8 * vec3((1. + sin(time  + 1.6))/2.,0,(1.+sin(time))/2.); return color;}vec3 buildingColor(vec3 position){if (time > 60)  return vec3(0.05,0.05,0.05); float eps = 0.000000001 ; float d = abs(position.y-city(position.xz) + .02) + eps; float inv = .02/d; vec3 couleur_neon = vec3((1. + sin(time  + 1.6))/2.,0,(1.+sin(time))/2.); if (time < 20) couleur_neon = vec3(1); return mix(vec3(0.05,0.05,0.05),couleur_neon,min(inv,1.8)); }vec3 skyColor(){ float y = (FragCoord.y+1.)/2.; vec3 haut = vec3(0.,0.,0.); vec3 bas = vec3(0.,0.,.5);      return mix(bas,haut,y);}vec4 rendu(vec2 p){ int id; vec3 n; vec3 posEye = camPos;vec4 p2 = invView*vec4(p.x/projMat.x, p.y/projMat.y, 1., 1.); p2 /= p2.w; vec3 rayDir = p2.xyz - posEye; rayDir = normalize(rayDir); vec3 rayPos = p2.xyz; vec3 intersection = sendRay(posEye, rayDir, id, n, rayPos); vec3 l = normalize(posEye-intersection) ; float dist = distance(intersection,posEye);vec3 color; if(id==0) color=groundColor(intersection); else if(id==1) color=buildingColor(intersection); else if(id==3) return vec4(mix(skyColor(),smokeColor,smokeCoeff),1.); if (dist < 1.) color = mix(color,1.8 * vec3(1.,0.5,0.),dist); vec3 finalcolor=color*dot(n,l);finalcolor = mix(finalcolor, smokeColor, smokeCoeff);return vec4(finalcolor,1.);}void main(){  if (((time-20) / 20) < 1){ smokeColor = vec3(1,1,1);} frag_color = rendu(FragCoord); if (frag_color.r == 0 && frag_color.g == 1 && frag_color.b == 1) frag_color = vec4(1,0,0,1); if (time > 100 + (1 - abs(FragCoord.y))*10)        frag_color = vec4(0,0,0,1);}";

    float time = 0.f;
    float oldTime = 0.f;

    //Pour la camera
    Camera cam(glm::vec3(0.f,0.f,0.f));

    init(strVertexShader, strFragmentShader);

    glViewport(0, 0, 800, 600);


    InputManager im;
    im.registerController(new SDL_Keyboard());
    do{
        e.getWindow()->clear();
/////////////////////////////////
        glClearColor(1.0f, 0.0f, 0.0f, 0.0f);
        glClear(GL_COLOR_BUFFER_BIT);


        time = (float)e.getTime()/1000.f;
        float deltaTime =time -oldTime;
        oldTime = time;
        //***********************************************
        // Calcul de la camera
        //***********************************************
        float tim = time / 10;
        float r = 0.8 + 1.6 * sin(6*tim);
        float theta = 2 * tim;
        float phi = 0.6 * M_PI * sin(12 * tim);

        float px = r * cos(phi) * cos(theta);
        float py = r * sin(phi);
        float pz = r * cos(phi) * sin(theta);

        //cam.Position = glm::vec3(-py / max(pz, 0.01), 0.7, px / max(pz, 0.01));

        float tim2 = time + 0.5;
        float r2 = 0.8 + 1.6 * sin(6*tim);
        float theta2 = 2 * tim;
        float phi2 = 0.6 * M_PI * sin(12 * tim);

        float px2 = r * cos(phi) * cos(theta);
        //cam.Target = glm::vec3(-pz2, py2, px2);

        // Camera avançant dans la ville
        float py2 = r * sin(phi);
        float pz2 = r * cos(phi) * sin(theta);

        //cam.Target = glm::vec3(-pz2, py2, px2);

        glm::vec3 speed(0, 0, (time - oldTime)*0.5);

        // Camera avancant dans la ville
        if ((int)(time/10) % 3 < 1){
        cam.Position = glm::vec3(sin(time), 1., time);
        cam.Target = glm::vec3(0, 1., time - 100);
        speed.x = (cam.Position.x - sin(oldTime))*0.5;

        GLuint sview_dist = glGetUniformLocation(theProgram, "view_dist");
        glUniform1f(sview_dist, 100);
        }
        else if ((int)(time/10) % 3 < 2)
        {
        // Camera survolant la ville
        cam.Position = glm::vec3(-2, 2.5, time);
        cam.Target = glm::vec3(90, 0, time - 100);

        GLuint sview_dist = glGetUniformLocation(theProgram, "view_dist");
        glUniform1f(sview_dist, 10);
        }
        // Camera Noeud
        else{
        cam.Position = posCam(time);
        cam.Target = posCam(time - deltaTime);
        //cam.Target.z *= -1;
        GLuint sview_dist = glGetUniformLocation(theProgram, "view_dist");
        glUniform1f(sview_dist, 10);
        }

        cam.Update(deltaTime);


        //***********************************************
        // Fin calcul de la camera
        //***********************************************


        // Draw the scene
        //glClearColor(0.0f, 0.0f, 0.0f, 0.0f);
        //glClear(GL_COLOR_BUFFER_BIT);

        glUseProgram(theProgram);

        // Mise a jour du temps dans le vertex shader
        GLuint stime = glGetUniformLocation(theProgram, "time");
        glUniform1f(stime, time);

        // Mise a jour de la distance de vue
        if (sin(time/3) > 0){
          GLuint sview_dist = glGetUniformLocation(theProgram, "view_dist");
          glUniform1f(sview_dist, 4.5);
        }
        else{
          GLuint sview_dist = glGetUniformLocation(theProgram, "view_dist");
          glUniform1f(sview_dist, 10);
        }

        // Mise a jour de la position de la camera
        float vec[3] = {cam.Position.x, cam.Position.y, cam.Position.z};
        GLuint camPos = glGetUniformLocation(theProgram, "camPos");
        glUniform3fv(camPos, 1, vec);

        // Mise a jour de la Inverse View Matrice
        glm::mat4x4 invViewMat = glm::inverseTranspose(cam.GetView());
        GLuint invView = glGetUniformLocation(theProgram, "invView");
        glUniformMatrix4fv(invView, 1, GL_TRUE, glm::value_ptr(invViewMat));


        // Mise a jour de la matrice de projection
        glm::mat4x4 temp = cam.GetProjection();
        float vec2[2] = { temp[0][0], temp[1][1]};
        GLuint projMat = glGetUniformLocation(theProgram, "projMat");
        glUniform2fv(projMat, 1, vec2);




        glBindBuffer(GL_ARRAY_BUFFER, positionBufferObject);
        glEnableVertexAttribArray(0);
        glVertexAttribPointer(0, 4, GL_FLOAT, GL_FALSE, 0, 0);

        glDrawArrays(GL_TRIANGLES, 0, 6);

        glDisableVertexAttribArray(0);
        glUseProgram(0);








////////////////////////////////

        e.getWindow()->refresh();
        im.update();
        e.waitFrame();

    }while(!im.needEscape() && time < 111.5);

    SDL_PauseAudio(1);
    ModPlug_Unload(Module);
    SDL_CloseAudio();

    return 0;
}
